//----------------------------------
//  piosender.v
//
//  11/08/31  ver 0.5   S.Nishida
//----------------------------------

`include "sa03board2_h.v"

/* note: hitdata is copied to buffer at least one clock after strobe.
 it is O.K. to use hd_trig signal as a strobe */

module piosender( clk, reset, strobe, pio_trigmask, pio_halfcycle,
                  indata, pck, pwrite, pdata );

input                         clk, reset, strobe;
input                         pio_trigmask;
input [`NB_PIO_HALFCYCLE-1:0] pio_halfcycle;
input [`NB_PIODATA-1:0]       indata;
output                        pck, pwrite;
output [`NB_PIO-1:0]          pdata;

reg                           pck, pwrite;

/* internal signal */
parameter [1:0] S_IDLE = 0, S_RESET = 1, S_PREPARE = 2, S_SEND = 3;

reg [1:0]                     state;
reg [`NB_CNT_PIO-1:0]         cnt;
reg [`NB_PIODATA-1:0]         buffer;
wire                          pio_rising, pio_falling;

/* main part */
clkgen #( .NB_HALFCYCLE(`NB_PIO_HALFCYCLE) )
   u0( .clk(clk), .reset(1'b0), .halfcycle(pio_halfcycle),
       .rising(pio_rising), .falling(pio_falling) );

always @(posedge clk or posedge reset) begin
    if( reset ) begin
        state <= S_IDLE;
        pwrite <= 1'b0;
    end
    else begin
        if( state==S_IDLE ) begin
            if( strobe ) state <= S_PREPARE;
        end
        else if( state==S_RESET ) begin
            if( !strobe ) begin
                if( pio_rising ) begin
                    state <= S_PREPARE;
                    pwrite <= 1'b0;
                end
            end
        end
        else if( state==S_PREPARE ) begin
            if( !pio_trigmask && strobe ) state <= S_RESET;
            else if( pio_rising ) begin
                state <= S_SEND;
                pwrite <= 1'b1;
                cnt <= `NCK_PIO-1;
            end
        end
        else begin // state == S_SEND
            if( !pio_trigmask && strobe ) state <= S_RESET;
            else if( pio_rising ) begin
                if( cnt != 0 ) cnt <= cnt - 1;
                else begin
                    state <= S_IDLE;
                    pwrite <= 1'b0;
                end
            end
        end
    end
end

always @(posedge clk or posedge reset) begin
    if( reset ) begin
        buffer <= 0;
    end
    else if( pio_rising ) begin
        if( state==S_PREPARE ) begin
            buffer <= indata;
        end
        else if( state==S_SEND ) begin
            buffer <= { buffer[`NB_PIODATA-`NB_PIO-1:0], { `NB_PIO{1'b0} } };
        end
    end
end

assign pdata = buffer[`NB_PIODATA-1:`NB_PIODATA-`NB_PIO];

always @(posedge clk) begin
    if( state==S_SEND ) begin
        if( pio_falling ) pck <= 1'b1;
        else if( pio_rising ) pck <= 1'b0;
    end
    else pck <= 1'b0;
end

endmodule
