//----------------------------------
//  ptm_ad5235.v
//
//  11/03/01  ver 0.2   S.Nishida
//----------------------------------

`include "sa03board2_h.v"

module ptm_ad5235( clk, reset, rising, falling, cmd_t,
                   cmd, data, rb_cmd, rb_data,
                   ptm_cs_, ptm_clk, ptm_sdi, ptm_wp_, ptm_pr_,
                   ptm_sdo );
// ptm_rdy not used at this moment

/* parameter */
parameter NB_SR = `NB_PTM_CMD + `NB_PTM_DATA;

input                     clk, reset, rising, falling, cmd_t;
input [`NB_PTM_CMD-1:0]   cmd;
input [`NB_PTM_DATA-1:0]  data;
output [`NB_PTM_CMD-1:0]  rb_cmd;
output [`NB_PTM_DATA-1:0] rb_data;
output 			  ptm_cs_, ptm_clk, ptm_sdi, ptm_wp_, ptm_pr_;
input 			  ptm_sdo;

reg 			  ptm_cs_, ptm_clk;

/* internal signal */
parameter [1:0] S_IDLE = 0, S_TRIG = 1, S_SET = 2;

reg [1:0] 		  state;
reg [`NB_CNT_PTM-1:0] 	  cnt;
reg [NB_SR-1:0] 	  sr, rbsr;

/* main */
assign { ptm_wp_, ptm_pr_ } = 2'b11;

always @(posedge clk or posedge reset) begin
    if( reset ) begin
        state <= S_IDLE;
        cnt <= 0;
        ptm_cs_ <= 1'b1;
    end
    else begin
        if( state==S_IDLE ) begin
            if( cmd_t ) state <= S_TRIG;
        end
        else if( state==S_TRIG ) begin
            if( falling ) begin
                state <= S_SET;
                cnt <= NB_SR-1; // <- NB_SR
                ptm_cs_ <= 1'b0;
            end
        end
        else if( state==S_SET ) begin
            if( falling ) begin
                if( cnt == 0 ) begin
                    state <= S_IDLE;
                    ptm_cs_ <= 1'b1;
                end
                else begin
                    cnt <= cnt - 1;
                end
            end
        end
    end
end


always @(posedge clk) begin
    if( rising && state==S_SET ) ptm_clk <= 1'b1;
    else if( falling ) ptm_clk <= 1'b0;
end

assign ptm_sdi = sr[NB_SR-1];
assign { rb_cmd, rb_data } = rbsr;

always @(posedge clk) begin
    if( state==S_TRIG ) sr <= { cmd, data };
    else if( falling && state==S_SET ) sr <= { sr[NB_SR-2:0], 1'b0 };
end

always @(posedge clk or posedge reset) begin
    if( reset ) rbsr <= 0;
    else begin
        if( rising && state==S_SET ) rbsr <= { rbsr[NB_SR-2:0], ptm_sdo };
    end
end

endmodule
