//----------------------------------
//  rbcp_sa03.v
//
//  12/10/04  ver 0.6   S.Nishida
//----------------------------------

`include "sa03board2_h.v"

module rbcp_sa03( clk, reset,
                  clear_t, runclear_t, init_t, sel_t, prmset_t, ndro_t,
                  softtrig_t, ts1_t, ts2_t,
                  ro_mode, polarity, lvds_trig, mode_edge, vetoen,
                  hdcycle, trgdelay, pio_trigmask, pio_halfcycle,
                  tp_fpga, tp_fpga1,
                  buf_selcid, buf_selch, buf_prm, prmrb,
                  veto, tx_busy,
                  ptm1_cmd_t, ptm1_cmd, ptm1_data, ptm1_rb_cmd, ptm1_rb_data,
                  ptm2_cmd_t, ptm2_cmd, ptm2_data, ptm2_rb_cmd, ptm2_rb_data,
                  mux_a, mux_b, mux_inh, ts1_data, ts2_data,
                  rbcp_act, rbcp_addr, rbcp_we, rbcp_wd, rbcp_re,
                  rbcp_rd, rbcp_ack );

input                          clk, reset;
output                         clear_t, runclear_t, init_t, sel_t, prmset_t;
output                         ndro_t, ts1_t, ts2_t, softtrig_t;
output                         ro_mode, polarity, lvds_trig, mode_edge, vetoen;
output [`NB_HDCYCLE-1:0]       hdcycle;
output [`NB_TRGDELAY-1:0]      trgdelay;
output                         pio_trigmask;
output [`NB_PIO_HALFCYCLE-1:0] pio_halfcycle;
output                         tp_fpga, tp_fpga1;
output [`NB_CHIP-1:0]          buf_selcid;
output [`NB_CHAN-1:0]          buf_selch;
output [`NB_PARAM_MAX-1:0]     buf_prm;
input [`NB_PARAM_MAX-1:0]      prmrb;
input                          veto, tx_busy;
output                         ptm1_cmd_t, ptm2_cmd_t;
output [`NB_PTM_CMD-1:0]       ptm1_cmd, ptm2_cmd;
output [`NB_PTM_DATA-1:0]      ptm1_data, ptm2_data;
input [`NB_PTM_CMD-1:0]        ptm1_rb_cmd, ptm2_rb_cmd;
input [`NB_PTM_DATA-1:0]       ptm1_rb_data, ptm2_rb_data;
input [`NB_TS_DATA-1:0]        ts1_data, ts2_data;
output                         mux_a, mux_b, mux_inh;

input                          rbcp_act;
input [`NB_RBCP_ADDR-1:0]      rbcp_addr;
input                          rbcp_we;
input [`NB_RBCP_WD-1:0]        rbcp_wd;
input                          rbcp_re;
output [`NB_RBCP_RD-1:0]       rbcp_rd;
output                         rbcp_ack;

reg                            ro_mode, polarity, lvds_trig, mode_edge, vetoen;
reg [`NB_HDCYCLE-1:0]          hdcycle;
reg [`NB_TRGDELAY-1:0]         trgdelay;
reg                            pio_trigmask;
reg [`NB_PIO_HALFCYCLE-1:0]    pio_halfcycle;
reg                            tp_fpga, tp_fpga1;
reg [`NB_PARAM_MAX-1:0]        buf_prm, buf_prmrb;
reg [`NB_CHIP-1:0]             buf_selcid;
reg [`NB_CHAN-1:0]             buf_selch;
reg [`NCHIP*`NB_CHAN-1:0]      buf_lastsel;
reg [`NB_PTM_CMD-1:0]          ptm1_cmd, ptm2_cmd;
reg [`NB_PTM_DATA-1:0]         ptm1_data, ptm2_data;
reg                            mux_a, mux_b, mux_inh;

/* internal signal */
wire [`NB_IADDR-1:0]           ia;
wire                           eq24, eq24w, ack_w;
reg                            we, re, p_we;
reg [`NB_RBCP_WD-1:0]          wd;
reg [`NB_RBCP_RD-1:0]          rd;
reg [`NB_RBCP_ADDR-1:0]        addr;
reg                            ack_r;
wire [`NB_RBCP_RD-1:0]         rd_val;

/* main part */
always @(posedge clk) begin
    if( rbcp_act ) begin
        we <= rbcp_we;
        re <= rbcp_re;
        if( rbcp_we | rbcp_re ) addr <= rbcp_addr;
        if( rbcp_we ) wd <= rbcp_wd;
    end
end

always @(posedge clk) p_we <= we;

assign eq24 = (addr[`NB_RBCP_ADDR-1:`NB_IADDR]==0);
assign eq24w = p_we & eq24;
assign ia = addr[`NB_IADDR-1:0];

/* write operation */
assign clear_t = ( eq24w & (ia==`ADDR_CLEAR) );
assign runclear_t = ( eq24w & (ia==`ADDR_RUNCLEAR) );
assign init_t = ( eq24w & (ia==`ADDR_INIT) );
assign softtrig_t = ( eq24w & (ia==`ADDR_SOFTTRIG) );
assign sel_t = ( eq24w & (ia==`ADDR_SEL) );
assign prmset_t = ( eq24w & (ia==`ADDR_PRMSET) );
assign ndro_t = ( eq24w & (ia==`ADDR_NDRO) );
assign ptm1_cmd_t = ( eq24w & (ia==`ADDR_PTM1_CMD) );
assign ptm2_cmd_t = ( eq24w & (ia==`ADDR_PTM2_CMD) );
assign ts1_t = ( eq24w & (ia==`ADDR_TS1_DATA0 || ia==`ADDR_TS1_DATA1) );
assign ts2_t = ( eq24w & (ia==`ADDR_TS2_DATA0 || ia==`ADDR_TS2_DATA1) );

assign ack_w = p_we;

always @(posedge clk or posedge reset) begin
    if( reset ) begin
        { buf_selcid, buf_selch } <= 0;
        buf_prm <= 0;
        pio_trigmask <= 1'b0;
        pio_halfcycle <= 3;
    end
    else begin
        if( we && eq24 ) begin
            if( ia==`ADDR_CSR1 )
              { ro_mode, polarity, lvds_trig, mode_edge, vetoen } <= wd[4:0];
            else if( ia==`ADDR_HDCYCLE ) hdcycle <= wd[`NB_HDCYCLE-1:0];
            else if( ia==`ADDR_TRGDELAY ) trgdelay <= wd[`NB_TRGDELAY-1:0];
            else if( ia==`ADDR_CSR2 )
              { pio_trigmask, pio_halfcycle } <= wd[`NB_PIO_HALFCYCLE:0];
            else if( ia==`ADDR_TP ) { tp_fpga1, tp_fpga } <= wd[1:0];
            else if( ia==`ADDR_SEL ) { buf_selcid, buf_selch } <= wd;
            else if( ia=={`ADDRBASE_WPARAM,2'h0} ) buf_prm[7:0] <= wd;
            else if( ia=={`ADDRBASE_WPARAM,2'h1} ) buf_prm[15:8] <= wd;
            else if( ia=={`ADDRBASE_WPARAM,2'h2} ) buf_prm[23:16] <= wd;
            else if( ia=={`ADDRBASE_WPARAM,2'h3} )
              buf_prm[`NB_PARAM_MAX-1:24] <= wd;
            else if( ia==`ADDR_PTM1_DATA0 ) ptm1_data[7:0] <= wd;
            else if( ia==`ADDR_PTM1_DATA1 ) ptm1_data[15:8] <= wd;
            else if( ia==`ADDR_PTM1_CMD ) ptm1_cmd <= wd;
            else if( ia==`ADDR_PTM2_DATA0 ) ptm2_data[7:0] <= wd;
            else if( ia==`ADDR_PTM2_DATA1 ) ptm2_data[15:8] <= wd;
            else if( ia==`ADDR_PTM2_CMD ) ptm2_cmd <= wd;
            else if( ia==`ADDR_MUX ) { mux_inh, mux_b, mux_a } <= wd[2:0];
        end
    end
end

/* lastsel is kept after write operation to ADDR_SEL */
always @(posedge clk or posedge reset) begin
    if( reset ) buf_lastsel <= 0;
    else begin
        if( sel_t ) begin
            if( buf_selcid==0 ) buf_lastsel[`NB_CHAN-1:0] <= buf_selch;
            else if( buf_selcid==1 )
              buf_lastsel[`NB_CHAN*2-1:`NB_CHAN] <= buf_selch;
            else if( buf_selcid==2 )
              buf_lastsel[`NB_CHAN*3-1:`NB_CHAN*2] <= buf_selch;
            else buf_lastsel[`NB_CHAN*4-1:`NB_CHAN*3] <= buf_selch;
        end
    end
end


/* read operation */

assign rd_val
  = (ia==`ADDR_STAT) ? { 6'h0, tx_busy, veto } :
    (ia==`ADDR_CSR1) ? { 3'h0, ro_mode, polarity,
                         lvds_trig, mode_edge, vetoen } :
    (ia==`ADDR_HDCYCLE) ? { {(`NB_RBCP_RD-`NB_HDCYCLE){1'b0}}, hdcycle } :
    (ia==`ADDR_TRGDELAY) ? { {(`NB_RBCP_RD-`NB_TRGDELAY){1'b0}}, trgdelay } :
    (ia==`ADDR_CSR2) ? { {(`NB_RBCP_RD-`NB_PIO_HALFCYCLE-1){1'b0}},
                         pio_trigmask, pio_halfcycle } :
    (ia==`ADDR_TP) ? { 6'h0, tp_fpga1, tp_fpga } :
    (ia==`ADDR_SEL) ? { buf_selcid, buf_selch } :
    (ia=={`ADDRBASE_LASTSEL,2'h0}) ?
    { {(`NB_RBCP_RD-`NB_CHAN){1'b0}}, buf_lastsel[`NB_CHAN-1:0] } :
    (ia=={`ADDRBASE_LASTSEL,2'h1}) ?
    { {(`NB_RBCP_RD-`NB_CHAN){1'b0}}, buf_lastsel[`NB_CHAN*2-1:`NB_CHAN] } :
    (ia=={`ADDRBASE_LASTSEL,2'h2}) ?
    { {(`NB_RBCP_RD-`NB_CHAN){1'b0}}, buf_lastsel[`NB_CHAN*3-1:`NB_CHAN*2] } :
    (ia=={`ADDRBASE_LASTSEL,2'h3}) ?
    { {(`NB_RBCP_RD-`NB_CHAN){1'b0}}, buf_lastsel[`NB_CHAN*4-1:`NB_CHAN*3] } :
    (ia=={`ADDRBASE_WPARAM,2'h0}) ? buf_prm[7:0] :
    (ia=={`ADDRBASE_WPARAM,2'h1}) ? buf_prm[15:8] :
    (ia=={`ADDRBASE_WPARAM,2'h2}) ? buf_prm[23:16] :
    (ia=={`ADDRBASE_WPARAM,2'h3}) ? buf_prm[`NB_PARAM_MAX-1:24] :
    (ia=={`ADDRBASE_RPARAM,2'h0}) ? prmrb[7:0] :
    (ia=={`ADDRBASE_RPARAM,2'h1}) ? prmrb[15:8] :
    (ia=={`ADDRBASE_RPARAM,2'h2}) ? prmrb[23:16] :
    (ia=={`ADDRBASE_RPARAM,2'h3}) ? prmrb[`NB_PARAM_MAX-1:24] :
    (ia==`ADDR_PTM1_DATA0) ? ptm1_data[7:0] :
    (ia==`ADDR_PTM1_DATA1) ? ptm1_data[15:8] :
    (ia==`ADDR_PTM1_CMD) ? ptm1_cmd :
    (ia==`ADDR_PTM1_RB_DATA0) ? ptm1_rb_data[7:0] :
    (ia==`ADDR_PTM1_RB_DATA1) ? ptm1_rb_data[15:8] :
    (ia==`ADDR_PTM1_RB_CMD) ? ptm1_rb_cmd :
    (ia==`ADDR_PTM2_DATA0) ? ptm2_data[7:0] :
    (ia==`ADDR_PTM2_DATA1) ? ptm2_data[15:8] :
    (ia==`ADDR_PTM2_CMD) ? ptm2_cmd :
    (ia==`ADDR_PTM2_RB_DATA0) ? ptm2_rb_data[7:0] :
    (ia==`ADDR_PTM2_RB_DATA1) ? ptm2_rb_data[15:8] :
    (ia==`ADDR_PTM2_RB_CMD) ? ptm2_rb_cmd :
    (ia==`ADDR_MUX) ? { 5'h0, mux_inh, mux_b, mux_a } :
    (ia==`ADDR_TS1_DATA0) ? ts1_data[7:0] :
    (ia==`ADDR_TS1_DATA1) ? ts1_data[15:8] :
    (ia==`ADDR_TS2_DATA0) ? ts2_data[7:0] :
    (ia==`ADDR_TS2_DATA1) ? ts2_data[15:8] :
    0;

always @(posedge clk) begin
    ack_r <= re;
    rd <= rd_val;
end

assign rbcp_rd = rd;

/* RBCP_ACK */
assign rbcp_ack = ack_r | ack_w;

endmodule
