//---------------------------------
//  sa03board2.v
//
//  12/10/04  ver 0.6   S.Nishida
//---------------------------------

`include "sa03board2_h.v"

module sa03board2( iclk, trig, pushb_, tp_fpga, tp_fpga1,
                   init_, selin, selck, wr, wck, din, dout,
                   dataout,
                   ptm1_cs_, ptm1_clk, ptm1_sdi, ptm1_wp_,
                   ptm1_pr_, ptm1_sdo, ptm1_rdy,
                   ptm2_cs_, ptm2_clk, ptm2_sdi, ptm2_wp_,
                   ptm2_pr_, ptm2_sdo, ptm2_rdy,
                   mux_a, mux_b, mux_inh,
                   ts1_cs_, ts1_sck, ts1_so,
                   ts2_cs_, ts2_sck, ts2_so,
                   sw,
                   pck_p, pck_n, pwrite_p, pwrite_n, pdata_p, pdata_n,
                   trig2_p, trig2_n, trig2out_p, trig2out_n,
                   ETH_RSTn, ETH_TX_CLK, ETH_TX_EN, ETH_TXD,
                   ETH_RX_CLK, ETH_RX_DV, ETH_RXD, ETH_RX_ER,
                   ETH_CRS, ETH_COL, ETH_MDC, ETH_MDIO );

input                  iclk, trig, pushb_;
output                 tp_fpga, tp_fpga1;
output [`NCHIP-1:0]    init_, selin, selck, wr, wck, din;
input [`NCHIP-1:0]     dout;
input [`NCH_BOARD-1:0] dataout;
output                 ptm1_cs_, ptm1_clk, ptm1_sdi, ptm1_wp_, ptm1_pr_;
input                  ptm1_sdo, ptm1_rdy;
output                 ptm2_cs_, ptm2_clk, ptm2_sdi, ptm2_wp_, ptm2_pr_;
input                  ptm2_sdo, ptm2_rdy;
output                 mux_a, mux_b, mux_inh;
output                 ts1_cs_, ts1_sck, ts2_cs_, ts2_sck;
input                  ts1_so, ts2_so;

input [3:0]            sw;
output                 pck_p, pck_n, pwrite_p, pwrite_n;
output [`NB_PIO-1:0]   pdata_p, pdata_n;
input                  trig2_p, trig2_n;
output                 trig2out_p, trig2out_n;
output                 ETH_RSTn;
input                  ETH_TX_CLK;  // in : Tx clock
output                 ETH_TX_EN;   // out  : Tx enable
output [3:0]           ETH_TXD;     // out  : Tx data[3:0]
input                  ETH_RX_CLK;  // in   : Rx clock
input                  ETH_RX_DV;   // in   : Rx data valid
input [3:0]            ETH_RXD;     // in   : Rx data[3:0]
input                  ETH_RX_ER;   // in   : Rx error
input                  ETH_CRS;     // in   : Carrier sense
input                  ETH_COL;     // in   : Collision detected
output                 ETH_MDC;     // out  : Clock for MDIO
inout                  ETH_MDIO;    // io   : Data

//--- internal signals ---
wire                   i_ETH_RX_CLK, i_ETH_TX_CLK;
wire                   GMII_MDIO_OE, GMII_MDIO_OUT;

//--- main part ---
wire                   pck, pwrite, trig2;
wire [`NB_PIO-1:0]     pdata;

IBUFG #(.IOSTANDARD("DEFAULT")) b00( .O(i_ETH_RX_CLK), .I(ETH_RX_CLK) );
IBUFG #(.IOSTANDARD("DEFAULT")) b01( .O(i_ETH_TX_CLK), .I(ETH_TX_CLK) );

n6_obuf_lvds33 b02( .O(pck_p), .OB(pck_n), .I(pck) );
n6_obuf_lvds33 b03( .O(pwrite_p), .OB(pwrite_n), .I(pwrite) );
n6_obuf_lvds33 b04( .O(pdata_p[0]), .OB(pdata_n[0]), .I(pdata[0]) );
n6_obuf_lvds33 b05( .O(pdata_p[1]), .OB(pdata_n[1]), .I(pdata[1]) );
n6_ibuf_lvds33 b06( .I(trig2_p), .IB(trig2_n), .O(trig2) );
n6_obuf_lvds33 b07( .O(trig2out_p), .OB(trig2out_n), .I(trig2out) );

assign  ETH_MDIO = (GMII_MDIO_OE ? GMII_MDIO_OUT : 1'bz);

wire [7:0]                 GMII_TXD, GMII_RXD;

assign ETH_TXD = GMII_TXD[3:0];
assign GMII_RXD = { 4'h0, ETH_RXD };

//----------------------------------------------------------------------
//      NETWORK PROTOCOL PROCESSOR
//----------------------------------------------------------------------
wire [31:0]                ext_ip_addr = { `IP_ADDR_28, sw };

wire [15:0]                TCP_RX_WC;
wire [`NB_TCP_RX_DATA-1:0] TCP_RX_DATA;
wire [`NB_TCP_TX_DATA-1:0] TCP_TX_DATA;

wire [`NB_RBCP_ADDR-1:0]   RBCP_ADDR;
wire [`NB_RBCP_WD-1:0]     RBCP_WD;
wire [`NB_RBCP_RD-1:0]     RBCP_RD;

my_wrapper_sitcp_xc6s_32k #(.TIM_PERIOD(25)) SiTCP(
  .CLK(iclk),             // in   : System Clock > 129MHz (for GMII)
  .RST(1'b0),             // in   : System reset ***
  // Configuration parameters
  .EXT_IP_ADDR(ext_ip_addr),    // in   : IP address[31:0]
  .EXT_TCP_PORT(16'd0),   // in   : TCP port #[15:0]
  .EXT_RBCP_PORT(16'd0),  // in   : RBCP port #[15:0]
  .PHY_ADDR(5'b00111),    // in   : PHY-device MIF address[4:0]
  // MII interface
  .GMII_RSTn(ETH_RSTn),   // out  : PHY reset
  .GMII_1000M(1'b0),      // in   : GMII mode (0:MII, 1:GMII) ***
  // TX
  .GMII_TX_CLK(i_ETH_TX_CLK),     // in   : Tx clock   ***
  .GMII_TX_EN(ETH_TX_EN), // out  : Tx enable
  .GMII_TXD(GMII_TXD[7:0]),       // out  : Tx data[7:0] ***
  .GMII_TX_ER(),          // out  : TX error ***
  // RX
  .GMII_RX_CLK(i_ETH_RX_CLK), // in   : Rx clock
  .GMII_RX_DV(ETH_RX_DV),     // in   : Rx data valid
  .GMII_RXD(GMII_RXD[7:0]),   // in   : Rx data[7:0] ***
  .GMII_RX_ER(ETH_RX_ER),     // in   : Rx error
  .GMII_CRS(ETH_CRS),         // in   : Carrier sense
  .GMII_COL(ETH_COL),         // in   : Collision detected
  // Management IF
  .GMII_MDC(ETH_MDC),             // out  : Clock for MDIO
  .GMII_MDIO_IN(ETH_MDIO),        // in   : Data
  .GMII_MDIO_OUT(GMII_MDIO_OUT),  // out  : Data
  .GMII_MDIO_OE(GMII_MDIO_OE),    // out  : MDIO output enable
  // User I/F
  .SiTCP_RST(SiTCP_RST),  // out  : Reset for SiTCP and related circuits
  // TCP connection control
  .TCP_OPEN_REQ(1'b0),    // in   : Reserved input, shoud be 0
  .TCP_OPEN_ACK(TCP_OPEN),  // out  : Acknowledge for open (=Socket busy)
  .TCP_ERROR(),   // out  : TCP error, its active period is equal to MSL
  .TCP_CLOSE_REQ(TCP_CLOSE),      // out  : Connection close request
  .TCP_CLOSE_ACK(TCP_CLOSE),      // in   : Acknowledge for closing
  // FIFO I/F
  .TCP_RX_WC(TCP_RX_WC[15:0]),    // in   : Rx FIFO write count[15:0] (Unused bits should be set 1)
  .TCP_RX_WR(TCP_RX_WR),          // out  : Write enable
  .TCP_RX_DATA(TCP_RX_DATA),      // out  : Write data[7:0]
  .TCP_TX_FULL(TCP_TX_FULL),      // out  : Almost full flag
  .TCP_TX_WR(TCP_TX_WR),          // in   : Write enable
  .TCP_TX_DATA(TCP_TX_DATA),      // in   : Write data[7:0]
  // RBCP
  .RBCP_ACT(RBCP_ACT),            // out  : RBCP active
  .RBCP_ADDR(RBCP_ADDR),          // out  : Address[31:0]
  .RBCP_WD(RBCP_WD),              // out  : Data[7:0]
  .RBCP_WE(RBCP_WE),              // out  : Write enable
  .RBCP_RE(RBCP_RE),              // out  : Read enable
  .RBCP_ACK(RBCP_ACK),            // in   : Access acknowledge
  .RBCP_RD(RBCP_RD)               // in   : Read data[7:0]
);

//----------------------------------------------------------------------
//      RBCP (Remote Bus Control Protocol)
//----------------------------------------------------------------------
wire                         clear_t, runclear_t, init_t, sel_t, prmset_t;
wire 			     ndro_t;
wire                         ro_mode, polarity, lvds_trig, mode_edge, vetoen;
wire [`NB_HDCYCLE-1:0]       hdcycle;
wire [`NB_TRGDELAY-1:0]      trgdelay;
wire                         pio_trigmask;
wire [`NB_PIO_HALFCYCLE-1:0] pio_halfcycle;
wire [`NB_CHIP-1:0]          buf_selcid;
wire [`NB_CHAN-1:0]          buf_selch;
wire [`NB_PARAM_MAX-1:0]     buf_prm, buf_prmrb;
wire                         veto, tx_busy;

wire                         ptm1_cmd_t, ptm2_cmd_t;
wire [`NB_PTM_CMD-1:0]       ptm1_cmd, ptm2_cmd, ptm1_rb_cmd, ptm2_rb_cmd;
wire [`NB_PTM_DATA-1:0]      ptm1_data, ptm2_data, ptm1_rb_data, ptm2_rb_data;
wire                         ts1_t, ts2_t;
wire [`NB_TS_DATA-1:0]       ts1_data, ts2_data;

rbcp_sa03 u0( .clk(iclk), .reset(SiTCP_RST),
              .clear_t(clear_t), .runclear_t(runclear_t), .init_t(init_t),
              .sel_t(sel_t), .prmset_t(prmset_t), .ndro_t(ndro_t),
	      .softtrig_t(softtrig_t), .ts1_t(ts1_t), .ts2_t(ts2_t),
              .ro_mode(ro_mode), .polarity(polarity), .lvds_trig(lvds_trig),
              .mode_edge(mode_edge), .vetoen(vetoen),
              .hdcycle(hdcycle), .trgdelay(trgdelay),
              .pio_trigmask(pio_trigmask), .pio_halfcycle(pio_halfcycle),
              .tp_fpga(tp_fpga), .tp_fpga1(tp_fpga1),
              .buf_selcid(buf_selcid), .buf_selch(buf_selch),
              .buf_prm(buf_prm), .prmrb(buf_prmrb),
              .veto(veto), .tx_busy(tx_busy),
              .ptm1_cmd_t(ptm1_cmd_t),
              .ptm1_cmd(ptm1_cmd), .ptm1_data(ptm1_data),
              .ptm1_rb_cmd(ptm1_rb_cmd), .ptm1_rb_data(ptm1_rb_data),
              .ptm2_cmd_t(ptm2_cmd_t),
              .ptm2_cmd(ptm2_cmd), .ptm2_data(ptm2_data),
              .ptm2_rb_cmd(ptm2_rb_cmd), .ptm2_rb_data(ptm2_rb_data),
              .mux_a(mux_a), .mux_b(mux_b), .mux_inh(mux_inh),
              .ts1_data(ts1_data), .ts2_data(ts2_data),
              .rbcp_act(RBCP_ACT), .rbcp_addr(RBCP_ADDR),
              .rbcp_we(RBCP_WE), .rbcp_wd(RBCP_WD), .rbcp_re(RBCP_RE),
              .rbcp_rd(RBCP_RD), .rbcp_ack(RBCP_ACK) );

//----------------------------------------------------------------------
//      user logic
//----------------------------------------------------------------------
wire                         sim_reset, rising, falling, i_init_, veto_fall;

assign sim_reset = 0;
assign reset = sim_reset | !i_init_;

assign mode_global = (buf_selch==0) ? 1'b1 : 1'b0;

clkgen #( .NB_HALFCYCLE(3)
          ) u10( .clk(iclk), .reset(1'b0), .halfcycle(3'b11),
                 .rising(rising), .falling(falling) );

init u11( .clk(iclk), .reset(1'b0), .init_t(init_t), .init_(i_init_) );
assign init_ = { `NCHIP{ i_init_ } };


saprmctl u12( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
              .sel_t(sel_t), .prmset_t(prmset_t), .ndro_t(ndro_t),
              .selcid(buf_selcid), .selch(buf_selch),
              .mode_global(mode_global), .prm(buf_prm), .rbsr(buf_prmrb),
              .rb_ready(),
              .selin(selin), .selck(selck),
              .wr(wr), .wck(wck), .din(din), .dout(dout) );

// selctl2 u12( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
//              .sel_t(sel_t), .selcid(buf_selcid), .selch(buf_selch),
//              .selin(selin), .selck(selck) );

// prmset2 u13( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
//              .prmset_t(prmset_t), .selcid(buf_selcid),
//              .mode_global(mode_global), .prm(buf_prm), .rbsr(buf_prmrb),
//              .wr(wr), .wck(wck), .din(din), .dout(dout), .rb_ready() );

wire                       exttrig, hd_shift, hd_trig;
wire [`NB_CNT_EVT-1:0]     cnt_evt, cnt_trg;
wire [`NB_HITDATA-1:0]     hitdata;
wire                       hd_trig_sitcp, hd_trig_pio;
wire [`NB_PIODATA-1:0]     indata_pio;

assign trig2out = trig2;
assign exttrig = ( lvds_trig ? trig2 : trig ) || !pushb_;

// always @(posedge iclk) p_veto <= veto;
// assign veto_fall = p_veto && !veto;
// assign clear = vetoen ? clear_t : veto_fall;
assign clear = clear_t;

trigctl u14( .clk(iclk), .reset(reset || runclear_t), .exttrig(exttrig),
             .softtrig_t(softtrig_t), .clear(clear), .vetoen(vetoen),
             .hdcycle(hdcycle), .trgdelay(trgdelay),
             .hd_shift(hd_shift), .veto(veto),
             .hd_trig(hd_trig), .cnt_evt(cnt_evt), .cnt_trg(cnt_trg) );

wire [`NCH_BOARD-1:0] data_tmp;
assign data_tmp = (polarity ? ~dataout : dataout );

hdsr144 u15( .clk(iclk), .reset(reset), .hd_shift(hd_shift), .hd_trig(hd_trig),
             .dataout(data_tmp), .mode_edge(mode_edge), .hitdata(hitdata) );

//----------------------------------------------------------------------
//      TCP
//----------------------------------------------------------------------
// assign TCP_RX_WC[15:11] = 5'b11111;
assign TCP_RX_WC = ~0;

// sitcp_fifo sitcp_fifo(
//         .clk(iclk), .rst(TCP_OPEN), .data_count(TCP_RX_WC[10:0]),
//         .full(), .wr_en(TCP_RX_WR), .din(TCP_RX_DATA[7:0]),
//         .empty(sitcpFifoEmpty), .rd_en(sitcpFifoRe),
//         .dout(TCP_TX_DATA[7:0]), .valid(TCP_TX_WR)
// );

// assign  sitcpFifoRe = ~TCP_TX_FULL & ~sitcpFifoEmpty;

assign hd_trig_sitcp = ro_mode ? 1'b0 : hd_trig;

tcpsender u16( .clk(iclk), .reset(reset), .hd_trig(hd_trig_sitcp),
               .hitdata(hitdata), .cnt_evt(cnt_evt), .cnt_trg(cnt_trg),
               .tcp_tx_full(TCP_TX_FULL), .tcp_tx_wr(TCP_TX_WR),
               .tcp_tx_data(TCP_TX_DATA), .tx_busy(tx_busy) );

// how to use TCP_OPEN ?

//----------------------------------------------------------------------
//      parallel IO
//----------------------------------------------------------------------
assign hd_trig_pio = ro_mode ? hd_trig : 1'b0;
assign indata_pio = { cnt_trg, hitdata };

piosender u17( .clk(iclk), .reset(reset), .strobe(hd_trig_pio),
               .pio_trigmask(pio_trigmask), .pio_halfcycle(pio_halfcycle),
               .indata(indata_pio), .pck(pck),
               .pwrite(pwrite), .pdata(pdata) );

//----------------------------------------------------------------------
//      peripherals
//----------------------------------------------------------------------
ptm_ad5235 u20( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
                .cmd_t(ptm1_cmd_t), .cmd(ptm1_cmd), .data(ptm1_data),
                .rb_cmd(ptm1_rb_cmd), .rb_data(ptm1_rb_data),
                .ptm_cs_(ptm1_cs_), .ptm_clk(ptm1_clk), .ptm_sdi(ptm1_sdi),
                .ptm_wp_(ptm1_wp_), .ptm_pr_(ptm1_pr_), .ptm_sdo(ptm1_sdo) );

ptm_ad5235 u21( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
                .cmd_t(ptm2_cmd_t), .cmd(ptm2_cmd), .data(ptm2_data),
                .rb_cmd(ptm2_rb_cmd), .rb_data(ptm2_rb_data),
                .ptm_cs_(ptm2_cs_), .ptm_clk(ptm2_clk), .ptm_sdi(ptm2_sdi),
                .ptm_wp_(ptm2_wp_), .ptm_pr_(ptm2_pr_), .ptm_sdo(ptm2_sdo) );

ts_tmp121 u23( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
               .ts_t(ts1_t), .data(ts1_data),
               .cs_(ts1_cs_), .sck(ts1_sck), .so(ts1_so) );

ts_tmp121 u24( .clk(iclk), .reset(reset), .rising(rising), .falling(falling),
               .ts_t(ts2_t), .data(ts2_data),
               .cs_(ts2_cs_), .sck(ts2_sck), .so(ts2_so) );

// monitor
// assign monout = { exttrig, dataout[108], dataout[72], dataout[36],
//                dataout[3:0], TCP_OPEN, TCP_TX_FULL, TCP_TX_WR,
//                tx_busy, hd_trig, softtrig_t, pushb_, iclk
//                };

// assign monout = { dataout[108], dataout[72], dataout[36],
//                TCP_OPEN, tx_busy, hd_trig, pushb_, iclk,
//                dataout[7:0] };

// assign monout = { ptm2_cmd_t, ptm2_cs_, ptm2_clk, ptm2_sdi, ptm2_rdy,
//                ptm2_sdo, ptm1_cmd_t, ptm1_cs_, ptm1_clk, ptm1_sdi,
//                ptm1_rdy, ptm1_sdo, iclk };

endmodule
