//----------------------------------
//  sa03board2_h.v
//
//  12/10/04  ver 0.6   S.Nishida
//----------------------------------

`ifndef SA03BOARD2_H
`define SA03BOARD2_H

/* IP address */
`define IP_ADDR_28      28'hc0a80a1 // 192.168.10.[16-31]

/* number of channel */
`define NCHIP            4  // #(chip) per board
`define NCH_SA02        36  // #(ch) per chip
`define NCH_BOARD      144  // #(ch) per board (=`NCH_SA01*`NCHIP)
`define NB_CHIP          2  // #(bit) to specify chip ID
`define NB_CHAN          6  // #(bit) to specify channel ID

/* data size */
`define NB_PARAM_MAX    26  // maximum of `NB_PARAM_GLOBAL and `NB_PARAM_CH
`define NB_PARAM_GLOBAL 26  // #(bit) of the global parameter
`define NB_PARAM_CH     18  // #(bit) of the parameter for each channel
`define NB_CNT_PARAM     5  // #(bit) to count `NB_PARAM_MAX

`define NB_HITDATA_CH    4  // #(bit) of hitdata per channel
`define NB_CNT_HITDATA  10  // #(bit) to count `NB_HITDATA
`define NB_HDSR          8

`define NB_HDCYCLE       5
`define NB_TRGDELAY      5

/* parameters common to sa02b2_pts_h */
`define NB_CNT_EVT      24  // #(bit) for the evtno/trigno counter.
`define NB_HITDATA     576  // #(bit) of hitdata (=`NCH_BOARD*`NB_DATA_CH)

/* parallel I/O data transfer (common to sa02b2_pts_h) */
`define NB_PIO           2
`define NB_PIOHEADER    24
`define NB_PIODATA      (`NB_PIOHEADER+`NB_HITDATA)
`define NCK_PIO         (`NB_PIODATA/`NB_PIO)
`define NB_CNT_PIO       9  // #(bit) to count NCK_PIO

`define NB_PIO_HALFCYCLE 3

/* SiTCP related */
`define NB_TCP_RX_DATA   8
`define NB_TCP_TX_DATA   8
`define NB_RBCP_ADDR    32
`define NB_RBCP_WD       8
`define NB_RBCP_RD       8

`define NB_HDHEADER     64  // #(bit) for hitdata header
`define NCK_TXDATA      ((`NB_HDHEADER+`NB_HITDATA)/`NB_TCP_TX_DATA)
//                             #(clk) to send TX data (hitdata+header)
`define NB_NCK_TXDATA    7  // #(bit) to count NCK_TXDATA

/* peripheral */
`define NB_PTM_CMD       8
`define NB_PTM_DATA     16
`define NB_CNT_PTM       5 // #(bit) to count `NB_PTM_CMD + `NB_PTM_DATA
`define NB_TS_DATA      16
`define NB_CNT_TS        5 // #(bit) to count `NB_TS_DATA

/* address A[7:0] */
`define NB_IADDR           8
`define ADDR_CLEAR         8'b0000_0000
`define ADDR_RUNCLEAR      8'b0000_0001
`define ADDR_STAT          8'b0000_0010 // { tx_busy, veto }
`define ADDR_CSR1          8'b0000_0011 // { ro_mode, 2'h0, mode_edge, vetoen }
`define ADDR_HDCYCLE       8'b0000_0100 // { hdcycle[5:0] }
`define ADDR_TRGDELAY      8'b0000_0101 // { trgdelay[5:0] }
`define ADDR_CSR2          8'b0000_0110 // { pio_trigmask, pio_halfcycle[2:0] }
`define ADDR_SOFTTRIG      8'b0000_0111
`define ADDR_TP            8'b0000_1000   // { tp }
`define ADDR_INIT          8'b0001_0000
`define ADDR_SEL           8'b0001_0001
`define ADDR_PRMSET        8'b0001_0010
`define ADDR_NDRO          8'b0001_0011
`define ADDRBASE_LASTSEL   6'b0001_01
`define ADDRBASE_WPARAM    6'b0010_00
`define ADDRBASE_RPARAM    6'b0010_01

`define ADDR_PTM1_DATA0    8'b0100_0000
`define ADDR_PTM1_DATA1    8'b0100_0001
`define ADDR_PTM1_CMD      8'b0100_0010
`define ADDR_PTM1_RB_DATA0 8'b0100_0100
`define ADDR_PTM1_RB_DATA1 8'b0100_0101
`define ADDR_PTM1_RB_CMD   8'b0100_0110

`define ADDR_PTM2_DATA0    8'b0101_0000
`define ADDR_PTM2_DATA1    8'b0101_0001
`define ADDR_PTM2_CMD      8'b0101_0010
`define ADDR_PTM2_RB_DATA0 8'b0101_0100
`define ADDR_PTM2_RB_DATA1 8'b0101_0101
`define ADDR_PTM2_RB_CMD   8'b0101_0110

`define ADDR_MUX           8'b0110_0000

`define ADDR_TS1_DATA0     8'b0110_1000
`define ADDR_TS1_DATA1     8'b0110_1001
`define ADDR_TS2_DATA0     8'b0110_1010
`define ADDR_TS2_DATA1     8'b0110_1011

`endif
