//----------------------------------
//  saprmctl.v
//
//  11/11/28   ver 0.1   S.Nishida
//----------------------------------

`include "sa03board2_h.v"

module saprmctl( clk, reset, rising, falling,
                 sel_t, prmset_t, ndro_t,
                 selcid, selch, mode_global, prm, rbsr, rb_ready,
                 selin, selck, wr, wck, din, dout );

/* parameter */
input                      clk, reset, rising, falling;
input                      sel_t, prmset_t, ndro_t;
input [`NB_CHIP-1:0]       selcid;
input [`NB_CHAN-1:0]       selch;
input                      mode_global;
input [`NB_PARAM_MAX-1:0]  prm;
output [`NB_PARAM_MAX-1:0] rbsr;
output                     rb_ready;
output [`NCHIP-1:0]        selin, selck, wr, wck, din;
input [`NCHIP-1:0]         dout;

reg [`NCHIP-1:0]           selin;
/* internal signal */
parameter [3:0] S_IDLE = 0, S_SEL_T = 1, S_PRMSET_T = 2, S_NDRO_T = 3,
             S_SEL_INIT = 4, S_SEL_SELECT = 5,
             S_PRMSET_INIT = 6, S_PRMSET_PRMSET = 7,
             S_NDRO_INIT = 8, S_NDRO_SELECT = 9, S_NDRO_PRMSET = 10,
				 S_RB_READY_T = 11;
reg [3:0]                  st;

reg                        i_din, i_selck, i_wck, i_wr;
reg                        c_rising, c_falling;
reg [`NB_CHAN-1:0]         sel_cnt;
reg [`NB_CNT_PARAM-1:0]    prm_cnt;
reg [`NB_PARAM_MAX-1:0]    prmsr, rbsr;

/* main */
always @(posedge clk) { c_rising, c_falling } <= { rising, falling };

// --- state ---
always @(posedge clk or posedge reset) begin
    if( reset ) st <= S_IDLE;
    else begin
        if( sel_t ) st <= S_SEL_T;
        else if( prmset_t ) st <= S_PRMSET_T;
        else if( ndro_t ) st <= S_NDRO_T;
		  else if( st == S_RB_READY_T ) st <= S_IDLE;
        else if( falling ) begin
            if( st == S_SEL_T ) st <= S_SEL_INIT;
            else if( st == S_NDRO_T ) st <= S_NDRO_INIT;
            else if( st == S_SEL_INIT ) begin
                if( sel_cnt == 0 ) st <= S_IDLE;
                else st <= S_SEL_SELECT;
            end
            else if( st == S_NDRO_INIT ) begin
                if( sel_cnt == 0 ) st <= S_NDRO_PRMSET;
                else st <= S_NDRO_SELECT;
            end
            else if( st == S_SEL_SELECT ) begin
                if( sel_cnt == 0 ) st <= S_IDLE;
            end
            else if( st == S_NDRO_SELECT ) begin
                if( sel_cnt == 0 ) st <= S_NDRO_PRMSET;
            end
            else if( st == S_NDRO_PRMSET ) begin
                if( prm_cnt == 0 ) st <= S_RB_READY_T;
            end
            else if( st == S_PRMSET_T ) st <= S_PRMSET_INIT;
            else if( st == S_PRMSET_INIT ) st <= S_PRMSET_PRMSET;
            else if( st == S_PRMSET_PRMSET ) begin
                if( prm_cnt == 0 ) st <= S_RB_READY_T;
            end
        end
    end
end

// --- selin ---
always @(posedge clk or posedge reset) begin
    if( reset ) selin <= 0;
    else begin
        if( st == S_SEL_INIT || st == S_NDRO_INIT ) begin
            selin[selcid] <= 1'b0;
        end
        else selin[selcid] <= 1'b1;
    end
end

// --- sel_cnt ---
always @(posedge clk or posedge reset) begin
    if( reset ) sel_cnt <= 0;
    else begin
        if( c_falling ) begin
            if( st == S_SEL_INIT || st == S_NDRO_INIT ) begin
                sel_cnt <= selch;
            end
            else if( st == S_SEL_SELECT || st == S_NDRO_SELECT ) begin
                if( sel_cnt != 0 ) sel_cnt <= sel_cnt - 1;
            end
        end
    end
end

// --- selck ---
always @(posedge clk or posedge reset) begin
    if( reset ) i_selck <= 1'b0;
    else begin
        if( st == S_SEL_SELECT || st == S_NDRO_SELECT ) begin
            if( c_rising ) i_selck <= 1'b1;
            else if( c_falling ) i_selck <= 1'b0;
        end
        else i_selck <= 1'b0;
    end
end

// --- prm_cnt ---
always @(posedge clk or posedge reset) begin
    if( reset ) prm_cnt <= 0;
    else begin
        if( c_falling ) begin
            if( st == S_PRMSET_INIT || st == S_NDRO_INIT ) begin
                if( mode_global ) prm_cnt <= `NB_PARAM_GLOBAL;
                else prm_cnt <= `NB_PARAM_CH;
            end
            else if( st == S_PRMSET_PRMSET || st == S_NDRO_PRMSET ) begin
                if( prm_cnt != 0 ) prm_cnt <= prm_cnt - 1;
            end
        end
    end
end

// --- wr ---
always @(posedge clk or posedge reset) begin
    if( reset ) i_wr <= 0;
    else begin
        if( st == S_PRMSET_PRMSET ) i_wr <= 1'b1;
        else i_wr <= 1'b0;
    end
end

// --- wck ---
always @(posedge clk or posedge reset) begin
    if( reset ) i_wck <= 0;
    else begin
        if( ( st == S_PRMSET_INIT && mode_global ) || st == S_NDRO_INIT
            || st == S_PRMSET_PRMSET || st == S_NDRO_PRMSET ) begin
            if( c_rising ) i_wck <= 1'b1;
            else if( c_falling ) i_wck <= 1'b0;
        end
        else i_wck <= 1'b0;
    end
end

// --- din, prmsr ---
always @(posedge clk or posedge reset) begin
    if( reset ) { i_din, prmsr } <= 0;
    else begin
        if( st == S_NDRO_INIT ) { i_din, prmsr } <= 0;
        else if( st == S_PRMSET_INIT ) begin
            if( mode_global ) begin
                prmsr[`NB_PARAM_MAX-1:`NB_PARAM_MAX-`NB_PARAM_GLOBAL]
                  <= prm[`NB_PARAM_GLOBAL-1:0];
            end
            else begin
                prmsr[`NB_PARAM_MAX-1:`NB_PARAM_MAX-`NB_PARAM_CH]
                  <= prm[`NB_PARAM_CH-1:0];
            end
        end
        else if( st == S_PRMSET_PRMSET ) begin
            if( c_falling ) { i_din, prmsr } <= { prmsr, 1'b0 };
        end
    end
end

// --- rbsr ---
always @(posedge clk or posedge reset) begin
    if( reset ) rbsr <= 0;
    else begin
        if( st == S_PRMSET_INIT || st == S_NDRO_INIT ) rbsr <= 0;
        else if( st == S_PRMSET_PRMSET || st == S_NDRO_PRMSET ) begin
            if( c_rising ) rbsr <= { rbsr[`NB_PARAM_MAX-2:0], dout[selcid] };
        end
    end
end

// ------------------------------------------------------
// always @(posedge clk) p_i_wr <= i_wr;
// assign rb_ready = p_i_wr && !i_wr;
assign rb_ready = (st == S_RB_READY_T);

function [`NCHIP-1:0] select_chip;
input [`NB_CHIP-1:0]     selcid;
input                    d;
integer                  i;
    begin
        select_chip = 0;
        for( i=0; i<`NCHIP; i=i+1 ) begin
            if( i==selcid ) select_chip[i] = d;
        end
    end
endfunction

assign selck = select_chip( selcid, i_selck );
assign wr = select_chip( selcid, i_wr );
assign wck = select_chip( selcid, i_wck );
assign din = select_chip( selcid, i_din );

endmodule
