//----------------------------------
//  tcpsender.v
//
//  10/10/21 ver 0.1  S.Nishida
//----------------------------------

`include "sa03board2_h.v"

module tcpsender( clk, reset, hd_trig,
                  hitdata, cnt_evt, cnt_trg,
                  tcp_tx_full, tcp_tx_wr, tcp_tx_data, tx_busy );

input                        clk, reset, hd_trig;
input [`NB_CNT_EVT-1:0]      cnt_evt, cnt_trg;
input [`NB_HITDATA-1:0]      hitdata;
input                        tcp_tx_full;
output                       tcp_tx_wr;
output [`NB_TCP_TX_DATA-1:0] tcp_tx_data;
output                       tx_busy;

/* internal signal */
parameter [1:0] S_IDLE = 0, S_TRIG = 1, S_SEND = 2, S_PAUSE = 3;

reg [`NB_HDHEADER+`NB_HITDATA-1:0] sr;
reg [1:0]                          state;
reg [`NB_NCK_TXDATA-1:0]           cnt;

/* main */
always @(posedge clk or posedge reset) begin
    if( reset ) begin
        state <= S_IDLE;
        cnt <= 0;
    end
    else begin
        if( state==S_IDLE ) begin
            if( hd_trig ) begin
                state <= S_TRIG;
            end
        end
        else if( state==S_TRIG ) begin
            cnt <= `NCK_TXDATA-1;
            if( tcp_tx_full ) state <= S_PAUSE;
            else state <= S_SEND;
        end
        else if( state==S_SEND ) begin
            if( cnt == 0 ) begin
                state <= S_IDLE;
            end
            else begin
                cnt <= cnt - 1;
                if( tcp_tx_full ) state <= S_PAUSE;
            end
        end
        else if( state==S_PAUSE ) begin
            if( !tcp_tx_full ) state <= S_SEND;
        end
    end
end


always @(posedge clk or posedge reset) begin
    if( reset ) begin
        sr <= 0;
    end
    else begin
        if( state==S_TRIG ) begin
            sr <= { hitdata, cnt_trg, cnt_evt, 16'h77c1 };
        end
        else if( state==S_SEND ) begin
            sr <= { {`NB_TCP_TX_DATA{1'b0}},
                    sr[`NB_HDHEADER+`NB_HITDATA-1:`NB_TCP_TX_DATA] };
        end
    end
end

assign tcp_tx_wr = (state==S_SEND);
assign tx_busy = (state!=S_IDLE);
assign tcp_tx_data = sr[`NB_TCP_TX_DATA-1:0];

endmodule
