//----------------------------------
//  trigctl.v
//
//  11/03/05  ver 0.2   S.Nishida
//----------------------------------

`include "sa03board2_h.v"

module trigctl( clk, reset, exttrig, softtrig_t, clear, vetoen,
                hdcycle, trgdelay,
                hd_shift, hd_trig, veto, cnt_evt, cnt_trg );

input                    clk, reset, exttrig, softtrig_t, clear, vetoen;
input [`NB_HDCYCLE-1:0]  hdcycle;
input [`NB_TRGDELAY-1:0] trgdelay;

output                   hd_shift, hd_trig, veto;
output [`NB_CNT_EVT-1:0] cnt_evt, cnt_trg;

reg [`NB_CNT_EVT-1:0]    cnt_evt, cnt_trg;

/* internal signal */
reg                      p_exttrig, c_exttrig, flag_trig, veto;
wire                     exttrig_edge, trig;
reg [`NB_HDCYCLE-1:0]    cnt;
reg [`NB_TRGDELAY-1:0]   trgcnt;

/* main */
always @(posedge clk) { p_exttrig, c_exttrig } <= { c_exttrig, exttrig };
assign exttrig_edge = !(p_exttrig) && c_exttrig;
assign trig = (exttrig_edge | softtrig_t) && !veto;

always @(posedge clk or posedge reset) begin
    if( reset ) begin
        cnt_evt <= 0;
        cnt_trg <= 0;
    end
    else begin
        if( exttrig_edge ) cnt_trg <= cnt_trg + 1;
        if( trig ) cnt_evt <= cnt_evt + 1;
    end
end

always @(posedge clk or posedge reset) begin
    if( reset ) veto <= 1'b0;
    else begin
        if( trig && vetoen ) veto <= 1'b1;
        else if( clear ) veto <= 1'b0;
    end
end

always @(posedge clk or posedge reset) begin
    if( reset ) cnt <= 1;
    else begin
        if( cnt==0 || cnt==1 ) cnt <= hdcycle;
        else cnt <= cnt - 1;
    end
end

assign hd_shift = (cnt==1);

always @(posedge clk) begin
    if( trig ) flag_trig <= 1'b1;
    else if( hd_shift ) flag_trig <= 1'b0;
end

always @(posedge clk) begin
    if( hd_shift ) begin
        if( trgdelay==0 ) trgcnt<=0;
        else begin
            if( trgcnt==0 && flag_trig ) trgcnt <= 1;
            else if( trgcnt==trgdelay ) trgcnt <= 0;
            else if( trgcnt!=0 ) trgcnt <= trgcnt + 1;
        end
    end
end

assign hd_trig = hd_shift && (trgdelay==0 ? flag_trig : (trgcnt==trgdelay));

endmodule
