//----------------------------------
//  ts_tmp121.v
//
//  11/10/20  ver 0.1   S.Nishida
//----------------------------------

`include "sa03board2_h.v"

module ts_tmp121( clk, reset, rising, falling, ts_t,
                  data, cs_, sck, so );

/* parameter */
input                    clk, reset, rising, falling, ts_t;
output [`NB_TS_DATA-1:0] data;
output                   cs_, sck;
input                    so;

reg [`NB_TS_DATA-1:0]    data;
reg                      cs_, sck;

/* internal signal */
parameter [1:0] S_IDLE = 0, S_TRIG = 1, S_SHIFT = 2;

reg [1:0]                st;
reg [`NB_CNT_PTM-1:0]    cnt;
reg                      c_rising, c_falling;

/* main */
always @(posedge clk) { c_rising, c_falling } <= { rising, falling };

always @(posedge clk or posedge reset) begin
    if( reset ) st <= S_IDLE;
    else begin
        if( ts_t ) st <= S_TRIG;
        else if( falling ) begin
            if( st == S_TRIG ) st <= S_SHIFT;
            else if( st == S_SHIFT ) begin
                if( cnt == 0 ) st <= S_IDLE;
            end
        end
    end
end

always @(posedge clk or posedge reset) begin
    if( reset ) cnt <= 0;
    else begin
        if( st == S_TRIG ) cnt <= `NB_TS_DATA; // no c_falling at S_TRIG!!
        else if( st == S_SHIFT ) begin
            if( c_falling ) begin
                if( cnt != 0 ) cnt <= cnt - 1;
            end
        end
    end
end

always @(posedge clk) cs_ = (st==S_SHIFT) ? 1'b0 : 1'b1;

always @(posedge clk or posedge reset) begin
    if( reset ) sck <= 1'b0;
    else begin
        if( st == S_IDLE ) sck <= 0;
        else if( st == S_SHIFT ) begin
            if( c_rising ) sck <= 1'b1;
            else if( c_falling ) sck <= 1'b0;
        end
    end
end

always @(posedge clk or posedge reset) begin
    if( reset ) data <= 0;
    else begin
        if( st == S_SHIFT && c_rising ) data <= { data[`NB_TS_DATA-2:0], so };
    end
end

endmodule
